package Datahub::Factory::Arthub;

use strict;
our $VERSION = '0.06';

1;
__END__

=encoding utf-8

=head1 NAME

Datahub::Factory::Arthub - modules for the L<VKC Arthub|>

=head1 DESCRIPTION

This modules contains packages that the L<VKC Arthub|> uses to extract data from project-specific sources.

It contains the following packages:

=over

=item L<Datahub::Factory::Importer::KMSKA>

=item L<Datahub::Factory::Importer::MSK>

=item L<Datahub::Factory::Module::PID>

=item L<Datahub::Factory::Importer::VKC>

=back

=head1 AUTHOR

Pieter De Praetere E<lt>pieter at packed.be E<gt>

=head1 COPYRIGHT

Copyright 2017- PACKED vzw

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=head1 SEE ALSO

L<Datahub::Factory>
L<Catmandu>

=cut